@@ -16,6 +16,13 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+/**
+ * SECTION: gva-columns
+ * @short_description: Column Creation and Persistence
+ *
+ * These functions manipulate the columns in the main tree view.
+ **/
+
 #ifndef GVA_COLUMNS_H
 #define GVA_COLUMNS_H
 
@@ -31,7 +38,8 @@ gboolean             gva_columns_lookup_id     (const gchar *column_name,
 const gchar *        gva_columns_lookup_name   (GvaGameStoreColumn column_id);
 void                 gva_columns_load          (GtkTreeView *view);
 void                 gva_columns_save          (GtkTreeView *view);
-gchar **             gva_columns_get_selected  (guint *length);
+GSList *             gva_columns_get_names     (GtkTreeView *view,
+                                                gboolean visible_only);
 
 G_END_DECLS
 
