@@ -16,6 +16,16 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+/**
+ * SECTION: gva-main
+ * @short_description: Main Window
+ *
+ * These functions manipulate the main window, excluding the tree view.
+ * The tree view functions are documented separately in the
+ * <link linkend="gnome-video-arcade-gva-tree-view">gva-tree-view</link>
+ * section.
+ **/
+
 #ifndef GVA_MAIN_H
 #define GVA_MAIN_H
 
@@ -29,7 +39,7 @@ GvaProcess *  gva_main_build_database           (GError **error);
 void          gva_main_connect_proxy_cb         (GtkUIManager *manager,
                                                  GtkAction *action,
                                                  GtkWidget *proxy);
-guint         gva_main_statusbar_get_context_id (const gchar *context);
+guint         gva_main_statusbar_get_context_id (const gchar *context_description);
 guint         gva_main_statusbar_push           (guint context_id,
                                                  const gchar *format,
                                                  ...);
@@ -37,6 +47,10 @@ void          gva_main_statusbar_pop            (guint context_id);
 void          gva_main_statusbar_remove         (guint context_id,
                                                  guint message_id);
 
+/* Signal Handlers */
+
+void          gva_main_window_destroy_cb        (GtkWindow *window);
+
 G_END_DECLS
 
 #endif /* GVA_MAIN_H */
