@@ -26,6 +26,7 @@
 #include "gva-favorites.h"
 #include "gva-game-store.h"
 #include "gva-main.h"
+#include "gva-search.h"
 #include "gva-ui.h"
 
 #define SQL_SELECT_GAMES \
@@ -191,7 +192,8 @@ gboolean
 gva_tree_view_update (GError **error)
 {
         const gchar *name;
-        const gchar *expr;
+        gchar *expr;
+        gboolean success;
 
         switch (gva_tree_view_get_selected_view ())
         {
@@ -200,18 +202,35 @@ gva_tree_view_update (GError **error)
                         break;
 
                 case 1:  /* Favorite Games */
-                        expr = "isfavorite(name) == \"yes\"";
+                        expr = g_strdup ("isfavorite(name) == \"yes\"");
                         break;
 
                 case 2:  /* Search Results */
-                        expr = "name == NULL";
+                {
+                        gchar *text;
+
+                        text = gva_search_get_last_search ();
+                        if (text != NULL && *text != '\0')
+                                expr = g_strdup_printf (
+                                        "name LIKE '%%%s%%' OR "
+                                        "description LIKE '%%%s%%' OR "
+                                        "manufacturer LIKE '%%%s%%' OR "
+                                        "year LIKE '%%%s%%'",
+                                        text, text, text, text);
+                        else
+                                expr = g_strdup ("name == NULL");
+                        g_free (text);
                         break;
+                }
 
                 default:
                         g_assert_not_reached ();
         }
 
-        if (!gva_tree_view_run_query (expr, error))
+        success = gva_tree_view_run_query (expr, error);
+        g_free (expr);
+
+        if (!success)
                 return FALSE;
 
         name = gva_tree_view_get_last_selected_game ();
@@ -245,6 +264,7 @@ gva_tree_view_run_query (const gchar *expr,
         GString *string;
         GSList *list;
         const gchar **strv;
+        gboolean sensitive;
         gchar *columns;
         gint ii = 0;
 
@@ -278,7 +298,8 @@ gva_tree_view_run_query (const gchar *expr,
         gdk_window_set_cursor (window, cursor);
         gtk_widget_set_sensitive (GTK_WIDGET (view), FALSE);
         model = gva_game_store_new_from_query (string->str, error);
-        gtk_widget_set_sensitive (GTK_WIDGET (view), TRUE);
+        sensitive = (gtk_tree_model_iter_n_children (model, NULL) > 0);
+        gtk_widget_set_sensitive (GTK_WIDGET (view), sensitive);
         gdk_window_set_cursor (window, NULL);
 
         gdk_cursor_unref (cursor);
@@ -297,6 +318,7 @@ gva_tree_view_run_query (const gchar *expr,
                 G_CALLBACK (tree_view_sort_column_changed_cb), NULL);
 
         gtk_tree_view_set_model (view, model);
+        gtk_tree_view_columns_autosize (view);
         g_object_unref (model);
 
         return TRUE;
@@ -425,8 +447,6 @@ gva_tree_view_get_selected_view (void)
  *
  * Sets the game list view corresponding to index @view.  See
  * gva_tree_view_get_selected_view() for a list of valid indices.
- * The function also calls gva_tree_view_set_last_selected_view()
- * so that the selected view will be persistent across sessions.
  *
  * Setting the game list view triggers a tree view update.  See
  * gva_tree_view_update() for details.
@@ -436,8 +456,6 @@ gva_tree_view_set_selected_view (gint view)
 {
         gtk_radio_action_set_current_value (
                 GTK_RADIO_ACTION (GVA_ACTION_VIEW_AVAILABLE), view);
-
-        gva_tree_view_set_last_selected_view (view);
 }
 
 /**
@@ -476,7 +494,7 @@ gva_tree_view_get_last_selected_game (void)
  * gva_tree_view_set_last_selected_game:
  * @game: the name of a game
  *
- * Writes %game to GConf key
+ * Writes @game to GConf key
  * <filename>/apps/gnome-video-arcade/selected-game</filename>.
  *
  * This is used to remember which game was selected in the previous
@@ -498,54 +516,6 @@ gva_tree_view_set_last_selected_game (const gchar *game)
         g_object_unref (client);
 }
 
-/**
- * gva_tree_view_get_last_selected_view:
- *
- * Returns the index of the most recently selected game list view in
- * either the current or the previous session of GNOME Video Arcade.
- *
- * Returns: the index of the most recently selected view
- **/
-gint
-gva_tree_view_get_last_selected_view (void)
-{
-        GConfClient *client;
-        gint view;
-        GError *error = NULL;
-
-        client = gconf_client_get_default ();
-        view = gconf_client_get_int (
-                client, GVA_GCONF_SELECTED_VIEW_KEY, &error);
-        gva_error_handle (&error);
-        g_object_unref (client);
-
-        return view;
-}
-
-/**
- * gva_tree_view_set_last_selected_view:
- * @view: the index of a game list view
- *
- * Writes @view to GConf key
- * <filename>/apps/gnome-video-arcade/selected-view</filename>.
- *
- * This is used to remember which game list view was selected in the
- * previous session of GNOME Video Arcade so that the same view can be
- * selected again automatically at startup.
- **/
-void
-gva_tree_view_set_last_selected_view (gint view)
-{
-        GConfClient *client;
-        GError *error = NULL;
-
-        client = gconf_client_get_default ();
-        gconf_client_set_int (
-                client, GVA_GCONF_SELECTED_VIEW_KEY, view, &error);
-        gva_error_handle (&error);
-        g_object_unref (client);
-}
-
 /**
  * gva_tree_view_get_last_sort_column_id:
  * @column_id: return location for the column ID
